import requests

from ._base import BaseImage


class Stripe(BaseImage):
    label = "stripe"
    name = "stripe"
    port = 12111                # HTTP port

    def check(self):
        url = f"http://{self.host}:{self.get_port()}/"
        try:
            resp = requests.get(url)
            if 200 <= resp.status_code < 500:
                return True
        except Exception:
            pass

        return False


stripe_image = Stripe()



from ._base import BaseImage

import requests


class EMQX(BaseImage):
    name = 'emqx'
    port = 1883

    def check(self):
        port = self.get_port(18083)
        url = f"http://{self.host}:{port}/status"
        try:
            resp = requests.get(url)
            if resp.status_code == 200:
                return True
        except Exception:
            return False


emqx_image = EMQX()



from ._base import BaseImage
from time import sleep


class Valkey(BaseImage):
    label = 'valkey'
    name = 'valkey'
    port = 6379

    def check(self):
        sleep(1)
        return True


valkey_image = Valkey()



from ._base import BaseImage
from time import sleep


class Redis(BaseImage):
    label = 'redis'
    name = 'redis'
    port = 6379

    def check(self):
        sleep(1)
        return True


redis_image = Redis()



from ._base import BaseImage

import requests


class InfluxDB(BaseImage):
    name = 'influxdb'
    port = 8086

    def check(self):
        port = self.get_port(8086)
        url = f"http://{self.host}:{port}/health"
        try:
            options = self.get_image_options()
            admin_user = options.get("environment", "admin").get("INFLUXDB_ADMIN_USER", "admin")  # noqa
            token = options["environment"]["INFLUXDB_ADMIN_USER_TOKEN"]
            session = requests.Session()
            session.auth = (admin_user, token)
            resp = requests.get(url)
            if resp.status_code == 200 and resp.json()["status"] == "pass":
                return True
        except Exception:
            return False


influxdb_image = InfluxDB()



from ._base import BaseImage

import os
import requests


class ElasticSearch(BaseImage):
    name = 'elasticsearch'
    port = 9200

    def get_image_options(self):
        image_options = super().get_image_options()
        if 'TRAVIS' in os.environ:
            image_options.update({
                'publish_all_ports': False,
                'ports': {
                    f'9200/tcp': '9200'
                }
            })
        return image_options

    def check(self):
        url = f'http://{self.host}:{self.get_port()}/'
        ssl_url = url.replace('http://', 'https://')
        try:
            resp = requests.get(url)
            if resp.status_code == 200:
                return True
        except Exception:
            try:
                # work with ssl
                resp = requests.get(
                    ssl_url, auth=('admin', 'admin'), verify=False)
                if resp.status_code == 200:
                    return True
            except Exception:
                pass
        return False


es_image = ElasticSearch()



from ._base import BaseImage

import random


image_name = 'test-etcd-{}'.format(random.randint(0, 1000))


class ETCD(BaseImage):
    name = 'etcd'
    port = 2379

    def get_image_options(self):
        image_options = super().get_image_options()
        image_options.update(dict(
            mem_limit='200m',
            name=image_name,
            command=' '.join([
                '/usr/local/bin/etcd',
                '--name {}'.format(image_name),
                '--data-dir /etcd-data',
                '--listen-client-urls http://0.0.0.0:2379',
                '--advertise-client-urls http://0.0.0.0:2379',
                '--listen-peer-urls http://0.0.0.0:2380',
                '--initial-advertise-peer-urls http://0.0.0.0:2380',
                '--initial-cluster {}=http://0.0.0.0:2380'.format(image_name),
                '--initial-cluster-token my-etcd-token',
                '--initial-cluster-state new',
                '--auto-compaction-retention 1'
            ])
        ))
        return image_options


etcd_image = ETCD()



from ._base import BaseImage


class RabbitMQ(BaseImage):
    label = 'rabbitmq'
    name = 'rabbitmq'
    port = 5672

    def check(self):
        import pika
        from pika.connection import URLParameters
        url = f'amqp://guest:guest@{self.host}:{self.get_port()}/%2F'
        try:
            connection = pika.BlockingConnection(URLParameters(url))
        except Exception:
            return False
        else:
            return connection.is_open


rabbitmq_image = RabbitMQ()



import platform

settings = {
    "cockroach": {
        "image": "cockroachdb/cockroach",
        "version": "v1.1.7",
        "options": {
            "command": " ".join(
                [
                    "start --insecure",
                ]
            ),
            "publish_all_ports": False,
            "ports": {"26257/tcp": "26257"},
        },
    },
    "elasticsearch": {
        "image": "elasticsearch",
        "version": "5.2.0",
        "env": {
            "cluster.name": "docker-cluster",
            "ES_JAVA_OPTS": "-Xms512m -Xmx512m",
            "xpack.security.enabled": "false",
        },
        "options": {"cap_add": ["IPC_LOCK"], "mem_limit": "1g"},
    },
    "etcd": {"image": "quay.io/coreos/etcd", "version": "v3.2.0-rc.0"},
    "kafka": {
        "image": "spotify/kafka",
        "version": "latest",
        "env": {"ADVERTISED_PORT": "9092", "ADVERTISED_HOST": "0.0.0.0"},
        "options": {"ports": {"9092": "9092", "2181": "2181"}},
    },
    "memcached": {
        "image": "memcached",
        "version": "1.6.7",
    },
    "minio": {
        "image": "minio/minio",
        "version": "latest",
        "env": {
            "MINIO_ACCESS_KEY": "x" * 10,
            "MINIO_SECRET_KEY": "x" * 10,
        },
        "options": {
            "cap_add": ["IPC_LOCK"],
            "mem_limit": "200m",
            "command": "server /export",
        },
    },
    "mysql": {
        "image": "mysql",
        "version": "5.7",
        "env": {"MYSQL_ALLOW_EMPTY_PASSWORD": "yes"},
    },
    "postgresql": {
        "image": "postgres",
        "version": "9.6.16",
        "env": {
            "POSTGRES_PASSWORD": "",
            "POSTGRES_DB": "guillotina",
            "POSTGRES_USER": "postgres",
        },
    },
    "rabbitmq": {"image": "rabbitmq", "version": "3.7.8"},
    "redis": {
        "image": "redis",
        "version": "7.0.10",
        "options": {"cap_add": ["IPC_LOCK"], "mem_limit": "200m"},
    },
    "valkey": {
        "image": "valkey/valkey",
        "version": "8.1.2",
        "options": {"cap_add": ["IPC_LOCK"], "mem_limit": "200m"},
    },
    "stripe": {
        "image": "stripe/stripe-mock",
        "version": "v0.136.0",
        "options": {
            "ports": {
                "12111": "12111",  # HTTP (default)
                "12112": "12112",  # HTTPS
            }
        },
    },
    "emqx": {
        "image": "emqx",
        "version": "5.5.1"
    },
    "influxdb": {
        "image": "bitnami/influxdb",
        "version": "2.7.5",  # V2
        "env": {
            "INFLUXDB_ADMIN_USER_PASSWORD": "adminadmin",
            "INFLUXDB_ADMIN_USER_TOKEN": "admin",  # Can be used as basic auth
            "INFLUXDB_USER_BUCKET": "foo-bucket",
            "INFLUXDB_DB": "foo-database",
            "INFLUXDB_ADMIN_ORG": "foo-org"
        }
    }
}


def get_image(name):
    image = settings[name]
    return image["image"] + ":" + image["version"]


def configure(
    name, image=None, version=None, full=None, env=None, options=None, max_wait_s=None
):
    if full is not None:
        image, _, version = full.partition(":")
    if image is not None:
        settings[name]["image"] = image
    if version is not None:
        settings[name]["version"] = version
    if options is not None:
        settings[name]["options"] = options
    if env is not None:
        if "env" not in settings[name]:
            settings[name]["env"] = {}
        settings[name]["env"].update(env)
    if max_wait_s:
        settings[name]["max_wait_s"] = max_wait_s


def get_env(name):
    image = settings[name]

    return image.get("env") or {}


def get_max_wait_s(name):
    # Default to 30 seconds
    image = settings[name]
    return image.get("max_wait_s") or 30


def get_options(name):
    image = settings[name]
    return image.get("options") or {}



from ._base import BaseImage


class Postgresql(BaseImage):
    name = 'postgresql'
    port = 5432

    def check(self):
        import psycopg2
        try:
            env = self.get_image_options()['environment']

            conn_string = f"dbname={env['POSTGRES_DB']} user={env['POSTGRES_USER']} " \
                f"host={self.host} port={self.get_port()}"
            if env.get('POSTGRES_PASSWORD'):
                conn_string += f" password={env['POSTGRES_PASSWORD']}"

            conn = psycopg2.connect(conn_string)
            cur = conn.cursor()
            cur.execute("SELECT 1;")
            cur.fetchone()
            cur.close()
            conn.close()
            return True
        except: # noqa
            conn = None
            cur = None
        return False


pg_image = Postgresql()



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup


setup(
    name='pytest-docker-fixtures',
    version='1.4.1',
    description='pytest docker fixtures',
    long_description=(open('README.rst').read() + '\n' +
                      open('CHANGELOG.rst').read()),
    keywords=['pytest', 'fixtures', 'docker'],
    author='Nathan Van Gheem',
    author_email='vangheem@gmail.com',
    classifiers=[
        'License :: OSI Approved :: BSD License',
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    url='https://github.com/guillotinaweb/pytest-docker-fixtures',
    license='BSD',
    zip_safe=True,
    include_package_data=True,
    # ext_modules=ext_modules,
    packages=find_packages(),
    install_requires=[
        'pytest',
        'docker',
        'requests'
    ],
    extras_require={
        'pg': [
            'psycopg2-binary'
        ],
        'mysql': [
            'mysql-connector-python~=8.0.17'
        ],
        'rabbitmq': [
            'pika==0.12.0'
        ],
        'kafka': [
            'kafka-python'
        ],
        "memcached": [
            "pymemcache"
        ],
    }
)



from ._base import BaseImage


class Memcached(BaseImage):
    label = 'memcached'
    name = 'memcached'
    port = 11211

    def check(self):
        local_port = self.get_port()

        from pymemcache.client.base import Client
        client = Client((self.host, local_port))
        try:
            server_stats = client.stats()
            return server_stats[b"accepting_conns"] == 1
        except (ConnectionRefusedError, KeyError):
            # not ready yet
            return False


memcached_image = Memcached()
